###*
# backbone.eventrouter - A highly opinionated, simplistic Backbone.Router coupled with a Backbone.Radio.Channel
# @version v1.0.1
# @link https://github.com/RoundingWellOS/backbone.eventrouter
# @license MIT
###

((global, factory) ->
  if typeof exports == 'object' and typeof module != 'undefined' then (module.exports = factory(require('underscore'), require('backbone'), require('backbone.radio'))) else if typeof define == 'function' and define.amd then define([
    'underscore'
    'backbone'
    'backbone.radio'
  ], factory) else global.Backbone = global.Backbone or {}
  global.Backbone.EventRouter = factory(global._, global.Backbone, global.Radio)
  return
) this, (_, Backbone, backbone_radio) ->
  'use strict'
  _ = if 'default' of _ then _['default'] else _
  Backbone = if 'default' of Backbone then Backbone['default'] else Backbone
  # eslint-disable-line
  # https://github.com/jashkenas/backbone/blob/1.2.1/backbone.js#L1425
  namedParamRegex = /(\(\?)?:\w+/

  ###*
  # Backbone.Router coupled with a Backbone.Radio Channel.
  #
  # @public
  # @class EventRouter
  # @memberOf Backbone
  ###

  EventRouter = Backbone.EventRouter = Backbone.Router.extend(
    constructor: (options) ->
      _.extend this, _.pick(options, [
        'channelName'
        'routeTriggers'
      ])
      @_ch = Backbone.Radio.channel(_.result(this, 'channelName'))
      @listenTo @_ch, 'all', @navigateFromEvent
      # Backbone.Router routes are added first
      # Routes can be added after the triggerRoutes with the Backbone.Router API
      Backbone.Router.apply this, arguments
      @_initRoutes()
      return
    channelName: 'event-router'
    getChannel: ->
      @_ch
    _initRoutes: ->
      @_routeTriggers = _.result(this, 'routeTriggers', {})
      _.each @_routeTriggers, @_addRouteTrigger, this
      return
    _addRouteTrigger: (routes, event) ->
      # handle any route as an array by default for the _.each
      routes = if _.isArray(routes) then routes else [ routes ]
      _.each routes, ((route) ->
        @route route, event, _.bind(@_ch.trigger, @_ch, event)
        return
      ), this
      return
    addRouteTrigger: (routes, event) ->
      @_routeTriggers[event] = routes
      @_addRouteTrigger routes, event
      this
    route: (_route, name, callback) ->
      bbRoute = Backbone.Router::route
      if _.isFunction(name) or !callback
        return bbRoute.call(this, _route, name, callback)
      wrappedCallback = _.bind((->
        args = _.drop(arguments, 0)
        # Trigger before: events that match currently triggered events
        @trigger 'before:route', name, args
        @trigger.apply this, [ 'before:route:' + name ].concat(args)
        @_storeRouteTrigger [ name ].concat(args)
        callback.apply this, args
        @_clearRouteTrigger()
        return
      ), this)
      bbRoute.call this, _route, name, wrappedCallback
    _storeRouteTrigger: (args) ->
      @_routeArgs = @_routeArgs or []
      @_routeArgs.push args
      return
    _getCurrentRouteTrigger: ->
      _.last(@_routeArgs) or []
    _clearRouteTrigger: ->
      @_routeArgs.pop()
      return
    _isTriggeredFromRoute: ->
      currentRoute = @_getCurrentRouteTrigger()
      if arguments.length != currentRoute.length
        return false
      arguments.length == _.union(arguments, @currentRoute).length
    navigateFromEvent: (event) ->
      route = @getDefaultRoute(event)
      # if no matching route exists do nothing
      if !route
        args = _.drop(arguments, 0)
        @trigger.apply this, [ 'noMatch' ].concat(args)
        return this
      if @_isTriggeredFromRoute.apply(this, arguments)
        return this
      eventArgs = _.drop(arguments, 1)
      translatedRoute = @translateRoute(route, eventArgs)
      # update URL without triggering the route
      @navigate translatedRoute, trigger: false
    getDefaultRoute: (event) ->
      routes = @_routeTriggers[event]
      if _.isArray(routes) then routes[0] else routes
    _replaceParam: (route, arg) ->
      route.replace namedParamRegex, arg
    translateRoute: (route, eventArgs) ->
      _.reduce eventArgs, @_replaceParam, route
  )
  EventRouter
## sourceMappingURL=backbone.eventrouter.js.map
