###*
* @license
* MarionetteJS (Marionette.AppRouter)
* ----------------------------------
* v1.0.1
*
* Copyright (c)2018 Derick Bailey, Muted Solutions, LLC.
* Distributed under MIT license
*
* http://marionettejs.com
###

import _ from 'lodash'
import { Router } from 'backbone'
import { MnObject } from 'backbone.marionette'

# App Router
# ----------
# Reduce the boilerplate code of handling route events
# and then calling a single method on another object,
# called a controller.
# Have your routers configured to call the method on
# your controller, directly.
#
# Configure an AppRouter with `appRoutes`.
#
# App routers can only take one `controller` object.
# It is recommended that you divide your controller
# objects in to smaller pieces of related functionality
# and have multiple routers / controllers, instead of
# just one giant router and controller.
#
# You can also add standard routes to an AppRouter.
# API borrowed from Marionette.Object
ObjectAPI = [
  'triggerMethod'
  'normalizeMethods'
  '_setOptions'
  'mergeOptions'
  'getOption'
  'bindEvents'
  'unbindEvents'
]
ClassOptions = [
  'appRoutes'
  'controller'
]

AppRouter = Router.extend
  constructor: (options) ->
    @_setOptions options
    @mergeOptions options, ClassOptions
    Router.apply this, arguments
    appRoutes = @appRoutes
    controller = @_getController()
    @processAppRoutes controller, appRoutes
    @on 'route', @_processOnRoute, this
    return
  # Similar to route method on a Backbone Router but
  # method is called on the controller
  appRoute: (route, methodName) ->
    controller = @_getController()
    @_addAppRoute controller, route, methodName
    return @
  # process the route event and trigger the onRoute
  # method call, if it exists
  _processOnRoute: (routeName, routeArgs) ->
    # make sure an onRoute before trying to call it
    if _.isFunction(@onRoute)
      # find the path that matches the current route
      routePath = _.invert(@appRoutes)[routeName]
      @onRoute routeName, routePath, routeArgs
    return
  # Internal method to process the `appRoutes` for the
  # router, and turn them in to routes that trigger the
  # specified method on the specified `controller`.
  processAppRoutes: (controller, appRoutes) ->
    if !appRoutes
      return this
    routeNames = _.keys(appRoutes).reverse()
    # Backbone requires reverted order of routes
    _.each routeNames, (route) =>
      @_addAppRoute controller, route, appRoutes[route]
      return
    return @
  _getController: ->
    return @controller
  _addAppRoute: (controller, route, methodName) ->
    method = controller[methodName]
    if !method
      throw new Error("Method #{methodName} was not found on the controller")
    @route route, methodName, _.bind(method, controller)
    return


_.extend AppRouter.prototype, _.pick(MnObject.prototype, ObjectAPI)

export default AppRouter
